function gadget:GetInfo()
	return {
		name      = "Scan Sweep",
		desc      = "Implements the Scan Sweep ability.",
		author    = "sprung",
		date      = "23/1/13",
		license   = "PD",
		layer     = 0,
		enabled   = true
	}
end

-- todo: smartcast

--include("LuaRules/Configs/customcmds.h.lua") -- zk doesnt have CMD_SCAN_SWEEP
local CMD_SCAN_SWEEP = 37496

local SCAN_RADIUS = 300

if (gadgetHandler:IsSyncedCode()) then

	-- constants
	local SCAN_TIME = 5*30
	local COOLDOWN_TIME = 60*30
	local SCAN_UNIT = "fakeunit"
	local SCAN_UNITDEF = UnitDefNames[SCAN_UNIT].id
	local scanner_names = {
		-- "comsat_station",
		"xelnaga_tower",
	}

	-- change names to IDs
	local scanners = {}
	for _, name in ipairs(scanner_names) do
		scanners[UnitDefNames[name].id] = true
	end
	scanner_names = nil

	-- performance
	local spInsertUnitCmdDesc = Spring.InsertUnitCmdDesc
	local spCreateUnit = Spring.CreateUnit
	local spDestroyUnit = Spring.DestroyUnit
	local spGetGameFrame = Spring.GetGameFrame
	local spFindUnitCmdDesc = Spring.FindUnitCmdDesc
	local spEditUnitCmdDesc = Spring.EditUnitCmdDesc
	local spSetUnitNeutral = Spring.SetUnitNeutral
	local spSetUnitBlocking = Spring.SetUnitBlocking
	local spSetUnitCollisionVolumeData = Spring.SetUnitCollisionVolumeData
	local spSetUnitSensorRadius = Spring.SetUnitSensorRadius
	local spSetUnitLosState = Spring.SetUnitLosState
	local spSetUnitLosMask = Spring.SetUnitLosMask
	local SendToUnsync = SendToUnsynced
	local spSetUnitNoSelect = Spring.SetUnitNoSelect
	local spSetUnitNoDraw = Spring.SetUnitNoDraw
	local spSetUnitNoMinimap = Spring.SetUnitNoMinimap
	local spSpawnCEG = Spring.SpawnCEG
	local ceil = math.ceil

	local scanSweepCmdDesc = {
		id      = CMD_SCAN_SWEEP,
		type    = CMDTYPE.ICON_MAP,
		name    = 'Scan',
		cursor  = 'Centroid',
		action  = 'scan_sweep',
		tooltip = 'Scan the selected location.',
		texture = 'bitmaps/scanner.png',
	}

	local scans = {}
	local cooldowns = {}

	local ally_count = #Spring.GetAllyTeamList() - 1

	function gadget:UnitCreated(unitID, unitDefID, unitTeam)
		if (scanners[unitDefID]) then
			spInsertUnitCmdDesc(unitID, scanSweepCmdDesc)
		end
	end

	function gadget:UnitPreDamaged(unitID, unitDefID, unitTeam, damage)
		if (unitID and scans[unitID]) then
			return 0 -- can't touch this!
		else
			return damage
		end
	end

	function gadget:UnitDestroyed(unitID, unitDefID, unitTeam)
		if (cooldowns[unitID]) then cooldowns[unitID] = nil end
		if (scans[unitID]) then scans[unitID] = nil end
	end

	function gadget:CommandFallback(unitID, unitDefID, teamID, cmdID, cmdParams, cmdOptions)
		if (cmdID ~= CMD_SCAN_SWEEP) then return false end
		if (cooldowns[unitID]) then return true, true end

		cooldowns[unitID] = spGetGameFrame() + COOLDOWN_TIME
		local editCmdDesc = { disabled = true }
		local cmdDescID = spFindUnitCmdDesc(unitID, CMD_SCAN_SWEEP)
		spEditUnitCmdDesc(unitID, cmdDescID, editCmdDesc)

		local scanID = spCreateUnit(SCAN_UNIT, cmdParams[1], cmdParams[2], cmdParams[3], 0, teamID)
		scans[scanID] = spGetGameFrame() + SCAN_TIME
		spSetUnitNeutral(scanID, true)
		spSetUnitBlocking(scanID, false, false, false)
		spSetUnitNoSelect (scanID, true)
		spSetUnitNoDraw (scanID, true)
		spSetUnitNoMinimap (scanID, true)
		spSetUnitSensorRadius(scanID, "los", SCAN_RADIUS)
		spSetUnitSensorRadius(scanID, "airLos", SCAN_RADIUS)
		spSetUnitSensorRadius(scanID, "radar", SCAN_RADIUS)
		spSetUnitSensorRadius(scanID, "sonar", SCAN_RADIUS)
		spSetUnitSensorRadius(scanID, "seismic", SCAN_RADIUS)
		spSetUnitSensorRadius(scanID, "radarJammer", 0)
		spSetUnitSensorRadius(scanID, "sonarJammer", 0)
		spSetUnitCollisionVolumeData(scanID
			, 0, 0, 0
			, 0, 0, 0
			, 0, 1, 0
		)

		for i = 1, ally_count do
			spSetUnitLosState(scanID, i, 0)
			spSetUnitLosMask (scanID, i, 15)
		end

		spSpawnCEG ("scan_sweep", cmdParams[1], cmdParams[2], cmdParams[3])
		_G.ScanSweep = scanID
		SendToUnsync("ScanSweep")
		_G.ScanSweep = nil
		return true, true
	end

	function gadget:GameFrame(n)
		if ((n % 30) ~= 0) then return end
		for id, timer in pairs(scans) do
			if (n > timer) then
				_G.ScanEnd = id
				SendToUnsync("ScanEnd")
				_G.ScanEnd = nil
				spDestroyUnit(id, false, true)
			end
		end
		for id, timer in pairs(cooldowns) do
			local editCmdDesc = {}
			if (n > timer) then
				editCmdDesc.disabled = false
				editCmdDesc.name = 'Scan'
				cooldowns[id] = nil
			else
				editCmdDesc.name = 'Scan (' .. ceil((timer - n) / 30) .. ')'
			end
			local cmdDescID = spFindUnitCmdDesc(id, CMD_SCAN_SWEEP)
			spEditUnitCmdDesc(id, cmdDescID, editCmdDesc)
		end
	end

else -- un/sync

	local spGetUnitPosition = Spring.GetUnitPosition
	local glColor = gl.Color
	local glDrawGroundCircle = gl.DrawGroundCircle

	local scans = {}
	local scancount = 0

	function gadget:Initialize()
		gadgetHandler:RegisterCMDID(CMD_SCAN_SWEEP)
		Spring.SetCustomCommandDrawData(CMD_SCAN_SWEEP, "Centroid", {0.0, 0.5, 1, 1})

		gadgetHandler:AddSyncAction("ScanSweep", function()
			local unitID = SYNCED.ScanSweep
			local xx, yy, zz = spGetUnitPosition(unitID)
			scancount = scancount + 1
			scans[unitID] = { x = xx, y = yy, z = zz }
		end)
		
		gadgetHandler:AddSyncAction("ScanEnd", function()
			local unitID = SYNCED.ScanEnd
			scans[unitID] = nil
		end)
	end

	function gadget:DrawWorldPreUnit()
		glColor(0.0, 0.5, 1.0, 0.5)
		for _, scan in pairs(scans) do
			glDrawGroundCircle(scan.x, scan.y, scan.z, SCAN_RADIUS, 32)
		end
		glColor(1,1,1,1)
	end
end